var YOGOAAT = YOGOAAT || {};

YOGOAAT.Actor = function() {
	this.name;
	this.handleDirection;
	this.lastGoodBoundingBox;

	this.boundingBox;
	this.hitBox;

	this.velocity = YOGOAAT.Vector.make2d(300, 800);

	this.jumpVelocity;
	this.jumpCompletion = -1;
	this.jumpQuantity;

	this.direction = YOGOAAT.Vector.make2d(1,0);

	this.canTake;
	this.taken;
	this.isTakeable = true;
	this.takenBy;

	this.bearerVelocityMultiplier = YOGOAAT.Vector.make2d(0.5, 0.5);
	this.bearerJumpQuantityMultiplier = 0.5;

	this.collisionHandled;

	//states
	this.isOnGround = false;
	this.isWalking = false;

	this.movePixels = function(vector, force) {

		if (this.takenBy && force != true) {
			return;
		} 

		this.boundingBox.x += vector.x; 
		this.boundingBox.y += vector.y; 

		if(this.taken) {
			this.taken.movePixels(vector, true);
		}

		

	};

	this.processDirection = function( vector ){

		if (vector.x == 0) {
			this.stay();
			return;
		}

		if (vector.x > 0) {
			this.direction.x = 1;
			this.direction.y = 0;
		} else {
			this.direction.x = -1;
			this.direction.y = 0;
		}

		this.walk();

	}

	this.move = function(vector, elapsedTime, force) {

		this.processDirection(vector);

		var velocity = this.velocity.clone();
		
		if (this.taken) {
			velocity.multiply(this.taken.bearerVelocityMultiplier);
		}

		var pixels = YOGOAAT.Vector.make2d(
			(elapsedTime * velocity.x * vector.x) / 1000,
			(elapsedTime * velocity.y * vector.y) / 1000
		);

		this.jumpCompletion += Math.abs(pixels.y);
		
		this.movePixels(pixels, force);
	};

	this.saveGoodPosition = function(){
		if(this.taken) {
			this.taken.saveGoodPosition();
		}

		this.lastGoodBoundingBox = this.boundingBox.clone();
	}

	this.restoreLastGoodPosition = function(){
		if(this.taken) {
			this.taken.restoreLastGoodPosition();
		}
		this.boundingBox = this.lastGoodBoundingBox.clone();
	}

	this.bound = function(box){
		var bounded = YOGOAAT.Box.make();
		bounded.x = this.boundingBox.x + box.x;
		bounded.y = this.boundingBox.y + box.y;
		bounded.width = box.width;
		bounded.height = box.height;
		return bounded;
	}

	this.walk = function(){
		this.isWalking = true;
	}

	this.stay = function(){
		this.isWalking = false;
	}

	this.getImageKey = function(){
		if (! this.handleDirections) {
			return this.name;
		}

		var action;
		action = this.isOnGround ? (this.isWalking ? "walk" : "stay") : "jump";

		var directions = {
			"0/1"  : "down",
			"0/-1" : "up",
			"1/0"  : "right",
			"-1/0" : "left"
		}

		var key = this.direction.x + "/" + this.direction.y;

		if (directions[key]) {
			return this.name + "-" + action + "-" + directions[key] ;
		}

		return this.name + "-" + action + "-right";
	}

	this.canTakeActor = function(actor) {
		if (! actor.isTakeable || this.taken) {
			return;
		}

		log(this.name + " can take " + actor.name);
		this.canTake = actor;
	}

	this.cannotTakeActor = function(){

		if (this.canTake && ! this.taken) {
			console.log(this.name + " cannot take " + this.canTake.name);
			this.canTake = false;

			this.collisionHandled = false;
		}

	}

	this.takeActor = function() {
		if (! this.canTake) {
			return false;
		}

		this.taken = this.canTake;
		this.taken.takenBy = this;
		this.canTake = false;

		this.taken.movePixels(YOGOAAT.Vector.make2d(0, -40), true);

		log(this.name + " has taken " + this.taken.name);
		return true;
	}

	this.dropActor = function() {

		if (! this.taken) {
			return false;
		}

		console.log(this.name + " drop " + this.taken.name);

		this.taken.takenBy = false;	
		this.taken = false;

		this.collisionHandled = false;

		return true;
		


	}

	this.initiateJump = function() {
		this.isOnGround = false;
		this.jumpCompletion = 0;
	}

	this.clearJump = function(){
		this.jumpCompletion = -1;
	}

	this.getCurrentJumpQuantity = function() {
		var value = this.jumpQuantity; 

		if (this.taken) {
			value *= this.taken.bearerJumpQuantityMultiplier;
		}

		return value;
	}

	this.processJumpQuantity = function(){

		if (this.jumpCompletion == -1) {
			return 0;
		}

		if(this.isOnGround) {
			return 0;
		}

		if (this.jumpCompletion >= this.getCurrentJumpQuantity()) {
			this.clearJump();
			return 0;
		}

		return -1;
		
	}
};
