var YOGOAAT = YOGOAAT || {};

YOGOAAT.Canvas2dLevelRenderer = function(canvasElement) {

	//public
	this.canvas = canvasElement;
	this.context;
	this.currentLevel;
	this.imageManager;

	this.debug = 0;

	this.prepare = function() {
		this.context = this.canvas.getContext("2d");
		this.currentLevel.camera.width = this.canvas.width;
		this.currentLevel.camera.height = this.canvas.height;
	}

	this.render = function(){

		this.clearCanvas();

		for (var i = 0; i < this.currentLevel.layers.bg.length; i++) {
			this.drawLayer(this.currentLevel.layers.bg[i].img);
		};

		if (this.debug) {
			this.drawLayer(this.currentLevel.mask);
		}

		for (var i = 0; i < this.currentLevel.actors.length; i++) {
			this.drawActor(this.currentLevel.actors[i]);
		};

		for (var i = 0; i < this.currentLevel.layers.fg.length; i++) {
			this.drawLayer(this.currentLevel.layers.fg[i].img);
		};
		
		
	};

	this.clearCanvas = function() {
		this.context.fillStyle = "white";
		this.context.fillRect(0,0, this.canvas.width, this.canvas.height);
	};

	this.drawLayer = function(image){

		var layerXPosition = this.layerPositionDimension(
			this.currentLevel.camera.x, 
			this.currentLevel.mask.width - image.width, 
			this.currentLevel.mask.width - this.currentLevel.camera.width
		);

		var layerYPosition = this.layerPositionDimension(
			this.currentLevel.camera.y, 
			this.currentLevel.mask.height - image.height, 
			this.currentLevel.mask.height - this.currentLevel.camera.height
		);

		var cameraPositionX = this.currentLevel.camera.x * -1;
		var cameraPositionY = this.currentLevel.camera.y * -1;

		this.context.drawImage(
			image, 
			cameraPositionX + layerXPosition,
			cameraPositionY + layerYPosition
		);
	};

	this.layerPositionDimension = function(cameraPosition, layerMax, cameraMax){
		var layerPostion = (cameraPosition * layerMax) / cameraMax;
		return layerPostion;
	}


	this.drawActor = function(actor){
		var cameraPositionX = this.currentLevel.camera.x * -1;
		var cameraPositionY = this.currentLevel.camera.y * -1;

		actorPositionX = cameraPositionX + actor.boundingBox.x;
		actorPositionY = cameraPositionY + actor.boundingBox.y;

		var sprite = this.imageManager.getImage("sprite", actor.getImageKey());
		this.context.drawImage(sprite, actorPositionX, actorPositionY);
		

		if(this.debug) {

			var colorBounding = actor.isOnGround ? "rgba(0,0,255,0.4)" : "rgba(0,255,0,0.4)";
			var colorHit = "rgba(255,0,0,0.4)";

			if (this.currentLevel.actors[0].canTake == actor) {
				colorBounding = "purple";
				colorHit = "orange";
			}

			if (actor.takenBy) {
				colorBounding = "rgba(0,0,255,0.2)";
				colorHit = "rgba(0,0,255,0.2)";
			}

			this.context.fillStyle = colorBounding;
			this.context.fillRect(
				actorPositionX, 
				actorPositionY, 
				actor.boundingBox.width, 
				actor.boundingBox.height
			);

			this.context.fillStyle = colorHit;
			this.context.fillRect(
				actorPositionX + actor.hitBox.x , 
				actorPositionY + actor.hitBox.y , 
				actor.hitBox.width, 
				actor.hitBox.height 
			);

		}
		

	};

};