var YOGOAAT = YOGOAAT || {};

YOGOAAT.GameController = function() {

	this.frameRate;
	this.animationFrameRate;

	this.levelLoader;
	this.levelRenderer;
	this.keyboardManager;
	this.soundManager;

	this.currentLevelIndex = 0;
	this.maxLevelIndex = 3;
	this.currentLevel;

	this.globalForce;
	this.frameGlobalForce;

	this.updateVolicityMax;
	this.updateThreshold;

	this.elapsedTime = 0;
	this.previousFrameTime = 0;

	this.cameraVelocity = 30;

	this.timer;
	this.timerAnim;

	this.start = function(){
		this.updateVolicityMax = 100;
		this.globalForce = YOGOAAT.Vector.make2d(0, 350);
		this.frameGlobalForce = YOGOAAT.Vector.make2d(0, 0);

		this.levelLoader.onLevelLoad = this.startLevel.bind(this);
		this.levelLoader.onLevelLoadError = function(){alert("Error Loading level !")};

		this.levelLoader.load(this.currentLevelIndex);
	};

	this.startLevel = function(level) {
		log("Level loaded");
		log(level);

		this.currentLevel = level;

		this.levelRenderer.currentLevel = level;
		this.levelRenderer.prepare();

		this.currentLevel.onFinish = this.startNextLevel.bind(this);
		
		this.previousFrameTime = Date.now();
		this.timer = window.setInterval(this.loop.bind(this), this.frameRate);
		this.timerAnim = window.setInterval(this.loopAnim.bind(this), this.animationFrameRate);

		this.soundManager.play("start");
	};

	this.startNextLevel = function(){
		
		if (this.currentLevelIndex + 1 <= this.maxLevelIndex) {
			this.soundManager.play("success");

			window.clearInterval(this.timer);
			window.clearInterval(this.timerAnim);

			this.currentLevelIndex ++;
			this.start();	
		}

	}

	this.loopAnim = function(){
		this.levelRenderer.imageManager.tick ++;
		/*
		if (this.levelRenderer.imageManager.tick > 60) {
			this.levelRenderer.imageManager.tick = 0;
			this.startNextLevel();
		}*/

	}



	this.loop = function(){
		log("loop");

		var now = Date.now();
  		this.elapsedTime = now - this.previousFrameTime;
  		this.previousFrameTime = now;

  		if (this.currentLevel.cameraPan) {
  			this.cameraAutoPan();	
  		} else {
  			this.cameraFollowHero();
  		}

		this.processGlobalForces();
		this.processMoves();
		this.processCollisions();

		this.levelRenderer.render();
	}

	this.cameraFollowHero = function() {
		
		this.currentLevel.camera.x = this.currentLevel.actors[0].boundingBox.x - this.currentLevel.camera.width / 2;
		this.currentLevel.camera.y = this.currentLevel.actors[0].boundingBox.y - this.currentLevel.camera.height / 2;

		if(this.currentLevel.camera.x > this.currentLevel.mask.width - this.currentLevel.camera.width) {
			this.currentLevel.camera.x = this.currentLevel.mask.width - this.currentLevel.camera.width;
		} 

		if(this.currentLevel.camera.x < 0) {
			this.currentLevel.camera.x = 0;
		} 

		if(this.currentLevel.camera.y > this.currentLevel.mask.height - this.currentLevel.camera.height) {
			this.currentLevel.camera.y = this.currentLevel.mask.height - this.currentLevel.camera.height;
		} 

		if(this.currentLevel.camera.y < 0) {
			this.currentLevel.camera.y = 0;
		} 

	}

	this.cameraAutoPan = function() {
		this.currentLevel.camera.x += (this.elapsedTime * this.cameraVelocity) / 1000;
		
		if(this.currentLevel.camera.x > this.currentLevel.mask.width - this.currentLevel.camera.width) {
			this.cameraVelocity = this.cameraVelocity * -1;
		} 

		if(this.currentLevel.camera.x < 0) {
			this.cameraVelocity = this.cameraVelocity * -1;
		} 

	}

	this.processCollisions = function(){
		this.updateThreshold = ((this.elapsedTime * this.updateVolicityMax) / 1000) * -1;
		this.updateThreshold -= this.frameGlobalForce.y;

		var nbActors = this.currentLevel.actors.length;

		for (var i = 0; i < nbActors; i++) {
			var actor = this.currentLevel.actors[i];
			this.processWorldCollision(actor);
		};

		for ( var coupleKey in this.currentLevel.actorsCouples ) {
			var couple = this.currentLevel.actorsCouples[coupleKey];
			var collisionUpdateNeeded = this.processActorCollision(couple.first, couple.second);
			
			if (collisionUpdateNeeded) {
				couple.first.movePixels(collisionUpdateNeeded);
				couple.first.isOnGround = true;
			}
			
			couple.first.saveGoodPosition();
			
		}

	};


	this.processWorldCollision = function (actor) {

		if (actor.takenBy) {
			return;
		}

		var updateNeeded = this.currentLevel.collideToWorld( actor );

		if (! updateNeeded) {
			actor.isOnGround = false;
			actor.saveGoodPosition();
			return;
		}

		actor.movePixels(updateNeeded);
		
		//too big
		if (updateNeeded.y < this.updateThreshold) {
			actor.restoreLastGoodPosition();
			log("updateNeeded too big. Limit : " + this.updateThreshold + " needed : " + updateNeeded.y);
		}  

		actor.isOnGround = true;
		actor.saveGoodPosition();
		
	}

	this.processActorCollision = function (actor1, actor2){

		if (actor1.takenBy || actor2.takenBy) {
			return;
		}

		if (actor1.takenBy == actor2 || actor2.takenBy == actor1 ) {
			return false;
		}

		var hitBox1 = actor1.bound(actor1.hitBox);
		var hitBox2 = actor2.bound(actor2.hitBox);

		var collideDimensionX1 = this.collideDimension(hitBox1.x, hitBox2.x, hitBox2.x + hitBox2.width);
		var collideDimensionY1 = this.collideDimension(hitBox1.y, hitBox2.y, hitBox2.y + hitBox2.height);
		
		var collideDimensionX2 = this.collideDimension(hitBox2.x, hitBox1.x, hitBox1.x + hitBox1.width);
		var collideDimensionY2 = this.collideDimension(hitBox2.y, hitBox1.y, hitBox1.y + hitBox1.height);

		var collide = (collideDimensionX1 || collideDimensionX2) && (collideDimensionY1 || collideDimensionY2);
		
		if ( ! collide ) {

			if (actor1.collisionHandled == actor2) {
				log("reset collision between " + actor1.name + " and " + actor2.name);
				actor1.cannotTakeActor();
				actor1.collisionHandled = false;
			}

			return false;
		}

		if (actor1.collisionHandled == actor2) {
			return YOGOAAT.Vector.make2d(0,0);
		}

		this.collisionHeroAndDropZone(actor1, actor2);

		var updateNeeded;
		log("Collision : " + actor1.name + " & " + actor2.name);

		if (collideDimensionY1) {
			updateNeeded = YOGOAAT.Vector.make2d(0, ( hitBox1.height + (hitBox1.y - hitBox2.y) ) * -1);	
		} 
		
		if (collideDimensionY2) {
			updateNeeded = YOGOAAT.Vector.make2d(0, ( (hitBox1.y + hitBox1.height) - hitBox2.y) * -1);	
		}
		

		if (actor1.name=="hero") {
			actor1.canTakeActor(actor2);
		} 

		actor1.collisionHandled = actor2;
		return updateNeeded;
	}

	this.collisionHeroAndDropZone = function(actor1, actor2) {
		var hero = (actor1.name == "hero") ? actor1 : (actor2.name == "hero") ? actor2 : false ;
		var zone = (actor1.name == "zone") ? actor1 : (actor2.name == "zone") ? actor2 : false ;

		if (! hero || ! zone) {
			return;
		}
		
		if (! hero.taken) {
			return;
		}

		this.soundManager.play("collect");

		var item = hero.taken;
		hero.dropActor();
		hero.clearJump();
		this.currentLevel.addCollected(item);

	}

	this.collideDimension = function(value, begin, end){
		if (value >= begin && value <= end) {
			return true;
		}

		return false;
	}	

	this.processMoves = function(){
		var actor = this.currentLevel.actors[0];
		var kb = this.keyboardManager;

		var move = YOGOAAT.Vector.make2d(0, 0);

		if(actor.isOnGround && kb.isDown(kb.UP)) {
			this.soundManager.play("jump");
			actor.initiateJump();
		}

		move.y += actor.processJumpQuantity();

		if(kb.isDown(kb.DOWN)) {
			move.y += 1;
		}

		if(kb.isDown(kb.LEFT)) {
			move.x += -1;
		}

		if(kb.isDown(kb.RIGHT)) {
			move.x += 1;
		}

		if(kb.isDown(kb.SPACE)) {
			if (actor.takeActor()){
				this.soundManager.play("take");
			}
			
		} else if (! kb.isDown(kb.SPACE)) {
			if (actor.dropActor()){
				this.soundManager.play("drop");
			}
		}

		actor.saveGoodPosition();
		actor.move(move, this.elapsedTime);


	}

	this.processGlobalForces = function() {
		
		this.frameGlobalForce.x = (this.elapsedTime * this.globalForce.x) / 1000;
		this.frameGlobalForce.y = (this.elapsedTime * this.globalForce.y) / 1000;

		for (var i = 0; i < this.currentLevel.actors.length; i++) {
			if (this.currentLevel.actors[i].isOnGround ) {
				continue;
			}

			this.currentLevel.actors[i].movePixels(this.frameGlobalForce);
		};
	}
};